/*
** Copyright (C) 2005-2010 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  Functions to support printing a record as text.
**
*/


#include <silk/silk.h>

RCSIDENT("$SiLK: rwascii.c 15604 2010-04-08 19:01:04Z mthomas $");

#include <silk/rwascii.h>
#include "skstream_priv.h"


/* how we know a field contains a callback */
#define RWASCII_CB_FIELD_ID  UINT32_MAX

/* error message when out of memory */
#define ERRMSG_ALLOC "Memory allocation failed at %s:%d", __FILE__, __LINE__


typedef struct rwascii_field_t {
    uint32_t                af_field_id;
    uint32_t                af_width;
    rwAsciiStreamGetValue_t af_cb_getvalue;
    rwAsciiStreamGetTitle_t af_cb_gettitle;
    void                   *af_cb_data;
} rwascii_field_t;


/* typedef rwAsciiStream_st rwAsciiStream_t; */
struct rwAsciiStream_st {
    FILE               *as_out_stream;
    rwascii_field_t    *as_field;
    uint32_t            as_field_count;
    uint32_t            as_field_capacity;
    uint32_t            as_timeflags;
    sk_ipv6policy_t     as_ipv6_policy;
    uint8_t             as_initialized;
    char                as_delimiter;
    unsigned            as_not_columnar     :1;
    unsigned            as_integer_ips      :1;
    unsigned            as_zero_pad_ips     :1;
    unsigned            as_no_titles        :1;
    unsigned            as_integer_sensors  :1;
    unsigned            as_no_final_delim   :1;
    unsigned            as_no_newline       :1;
    unsigned            as_legacy_icmp      :1;
    unsigned            as_field_icmp       :1;
};


/*
 * This struct holds the field names and their IDs.  The same names
 * are used for the column titles.
 *
 * Names that map to the same ID must be put together, with the name
 * that you want to use for the title first, then any aliases
 * afterward.

 * NOTE!! We are assuming that the stringmap code leaves things in the
 * order we insert them and doesn't re-arrange them.  Since that code
 * uses a linked-list, we are safe for now.
 */
static const sk_stringmap_entry_t field_map_entries[] = {
    {"sIP",          RWREC_FIELD_SIP,               NULL, NULL},
    {"1",            RWREC_FIELD_SIP,               NULL, NULL},
    {"dIP",          RWREC_FIELD_DIP,               NULL, NULL},
    {"2",            RWREC_FIELD_DIP,               NULL, NULL},
    {"sPort",        RWREC_FIELD_SPORT,             NULL, NULL},
    {"3",            RWREC_FIELD_SPORT,             NULL, NULL},
    {"dPort",        RWREC_FIELD_DPORT,             NULL, NULL},
    {"4",            RWREC_FIELD_DPORT,             NULL, NULL},
    {"protocol",     RWREC_FIELD_PROTO,             NULL, NULL},
    {"5",            RWREC_FIELD_PROTO,             NULL, NULL},
    {"packets",      RWREC_FIELD_PKTS,              NULL, NULL},
    {"pkts",         RWREC_FIELD_PKTS,              NULL, NULL},
    {"6",            RWREC_FIELD_PKTS,              NULL, NULL},
    {"bytes",        RWREC_FIELD_BYTES,             NULL, NULL},
    {"7",            RWREC_FIELD_BYTES,             NULL, NULL},
    {"flags",        RWREC_FIELD_FLAGS,             NULL, NULL},
    {"8",            RWREC_FIELD_FLAGS,             NULL, NULL},
    {"sTime",        RWREC_FIELD_STIME,             NULL, NULL},
    {"9",            RWREC_FIELD_STIME,             NULL, NULL},
    {"dur",          RWREC_FIELD_ELAPSED,           NULL, NULL},
    {"10",           RWREC_FIELD_ELAPSED,           NULL, NULL},
    {"eTime",        RWREC_FIELD_ETIME,             NULL, NULL},
    {"11",           RWREC_FIELD_ETIME,             NULL, NULL},
    {"sensor",       RWREC_FIELD_SID,               NULL, NULL},
    {"12",           RWREC_FIELD_SID,               NULL, NULL},
    {"in",           RWREC_FIELD_INPUT,             NULL, NULL},
    {"13",           RWREC_FIELD_INPUT,             NULL, NULL},
    {"out",          RWREC_FIELD_OUTPUT,            NULL, NULL},
    {"14",           RWREC_FIELD_OUTPUT,            NULL, NULL},
    {"nhIP",         RWREC_FIELD_NHIP,              NULL, NULL},
    {"15",           RWREC_FIELD_NHIP,              NULL, NULL},
    {"initialFlags", RWREC_FIELD_INIT_FLAGS,        NULL, NULL},
    {"26",           RWREC_FIELD_INIT_FLAGS,        NULL, NULL},
    {"sessionFlags", RWREC_FIELD_REST_FLAGS,        NULL, NULL},
    {"27",           RWREC_FIELD_REST_FLAGS,        NULL, NULL},
    {"attributes",   RWREC_FIELD_TCP_STATE,         NULL, NULL},
    {"28",           RWREC_FIELD_TCP_STATE,         NULL, NULL},
    {"application",  RWREC_FIELD_APPLICATION,       NULL, NULL},
    {"29",           RWREC_FIELD_APPLICATION,       NULL, NULL},
    {"class",        RWREC_FIELD_FTYPE_CLASS,       NULL, NULL},
    {"20",           RWREC_FIELD_FTYPE_CLASS,       NULL, NULL},
    {"type",         RWREC_FIELD_FTYPE_TYPE,        NULL, NULL},
    {"21",           RWREC_FIELD_FTYPE_TYPE,        NULL, NULL},
    {"sTime+msec",   RWREC_FIELD_STIME_MSEC,        NULL, NULL},
    {"22",           RWREC_FIELD_STIME_MSEC,        NULL, NULL},
    {"eTime+msec",   RWREC_FIELD_ETIME_MSEC,        NULL, NULL},
    {"23",           RWREC_FIELD_ETIME_MSEC,        NULL, NULL},
    {"dur+msec",     RWREC_FIELD_ELAPSED_MSEC,      NULL, NULL},
    {"24",           RWREC_FIELD_ELAPSED_MSEC,      NULL, NULL},
    {"icmpTypeCode", RWREC_FIELD_ICMP_TYPE_CODE,    NULL, NULL},
    {"25",           RWREC_FIELD_ICMP_TYPE_CODE,    NULL, NULL}
};

static const size_t field_map_count = (sizeof(field_map_entries)
                                       / sizeof(sk_stringmap_entry_t));


static int rwAsciiAllocFields(rwAsciiStream_t *astream, uint32_t capacity)
{
    void *current_fields;

    if (capacity > 0) {
        if (capacity < astream->as_field_capacity) {
            return 0;
        }
    } else if (astream->as_field_capacity == 0) {
        capacity = field_map_count;
    } else {
        capacity = 2 * astream->as_field_capacity;
    }

    if (NULL == astream->as_field) {
        assert(0 == astream->as_field_capacity);
        astream->as_field = calloc(capacity, sizeof(rwascii_field_t));
        if (NULL == astream->as_field) {
            return -1;
        }
        astream->as_field_capacity = capacity;
        return 0;
    }

    assert(astream->as_field_capacity > 0);

    current_fields = astream->as_field;
    astream->as_field = realloc(astream->as_field,
                                (capacity * sizeof(rwascii_field_t)));
    if (NULL == astream->as_field) {
        astream->as_field = current_fields;
        return -1;
    }

    memset(astream->as_field + astream->as_field_capacity, 0,
           (capacity - astream->as_field_capacity) * sizeof(rwascii_field_t));
    astream->as_field_capacity = capacity;
    return 0;
}


/*
**  rwAsciiSetWidths(astream);
**
**    Set the column widths for all the columns in the 'astream'
**    struct.  The column widths will depend on flags set by the user,
**    such as whether the output is columnar, or the form of sensors
**    (names or numbers).
*/
static void rwAsciiSetWidths(rwAsciiStream_t *astream)
{
    rwascii_field_t *field;
    uint32_t i;

    if (astream->as_not_columnar) {
        return;
    }

    for (i = 0, field = astream->as_field;
         i < astream->as_field_count;
         ++i, ++field)
    {
        if (RWASCII_CB_FIELD_ID == astream->as_field[i].af_field_id) {
            /* callback field. we already have the width */
            continue;
        }
        switch ((rwrec_printable_fields_t)field->af_field_id) {
          case RWREC_FIELD_SIP:
          case RWREC_FIELD_DIP:
          case RWREC_FIELD_NHIP:
            /* ip numbers */
#if SK_ENABLE_IPV6
            if (astream->as_ipv6_policy >= SK_IPV6POLICY_MIX) {
                /* If IPv6 support is available and IPv6 flows are not
                 * being ignored */
                field->af_width = 39;
                break;
            }
#endif /* SK_ENABLE_IPV6 */
            if (astream->as_integer_ips) {
                field->af_width = 10;
            } else {
                field->af_width = 15;
            }
            break;

          case RWREC_FIELD_SPORT:
          case RWREC_FIELD_DPORT:
            /* sport and dport */
            field->af_width = 5;
            break;

          case RWREC_FIELD_PROTO:
            /* proto */
            field->af_width = 3;
            break;

          case RWREC_FIELD_PKTS:
          case RWREC_FIELD_BYTES:
            /* packets, bytes */
            field->af_width = 10;
            break;

          case RWREC_FIELD_FLAGS:
          case RWREC_FIELD_INIT_FLAGS:
          case RWREC_FIELD_REST_FLAGS:
          case RWREC_FIELD_TCP_STATE:
            /* tcp flags, init-flags, non-init-flags, tcp-state */
            field->af_width = 8;
            break;

          case RWREC_FIELD_APPLICATION:
            field->af_width = 5;
            break;

          case RWREC_FIELD_ELAPSED:
            /* elapsed/duration */
            if (astream->as_timeflags & SKTIMESTAMP_NOMSEC) {
                field->af_width = 5;
                break;
            }
            /* else fallthrough */

          case RWREC_FIELD_ELAPSED_MSEC:
            /* elapsed/duration with milliseconds */
            field->af_width = 9;
            break;

          case RWREC_FIELD_STIME:
          case RWREC_FIELD_ETIME:
            /* sTime and end time */
            if (astream->as_timeflags & SKTIMESTAMP_NOMSEC) {
                if (astream->as_timeflags & SKTIMESTAMP_EPOCH) {
                    field->af_width = 10;
                } else {
                    field->af_width = 19;
                }
                break;
            }
            /* else fallthrough */

          case RWREC_FIELD_STIME_MSEC:
          case RWREC_FIELD_ETIME_MSEC:
            /* sTime and end time with milliseconds */
            if (astream->as_timeflags & SKTIMESTAMP_EPOCH) {
                field->af_width = 14;
            } else {
                field->af_width = 23;
            }
            break;

          case RWREC_FIELD_SID:
            /* sensor */
            if (astream->as_integer_sensors) {
                field->af_width = 5;
            } else {
                field->af_width = (uint8_t)sksiteSensorGetMaxNameStrLen();
            }
            break;

          case RWREC_FIELD_INPUT:
          case RWREC_FIELD_OUTPUT:
            /* input,output */
            field->af_width = 5;
            break;

          case RWREC_FIELD_FTYPE_CLASS:
            /* flow-type class */
            field->af_width = (uint8_t)sksiteClassGetMaxNameStrLen();
            break;

          case RWREC_FIELD_FTYPE_TYPE:
            /* flow-type type */
            field->af_width = (uint8_t)sksiteFlowtypeGetMaxTypeStrLen();
            break;

          case RWREC_FIELD_ICMP_TYPE_CODE:
            /* ICMP type and code in two columns */
            field->af_width = 7;
            break;

        } /* switch */
    }
    return;
}


/*
 *  rwAsciiSetDefaultFields(astream);
 *
 *    Configure the 'astream' to print all fields.
 */
static void rwAsciiSetDefaultFields(rwAsciiStream_t *astream)
{
    uint32_t i;

    assert(astream->as_field == NULL);

    astream->as_field_count = RWREC_PRINTABLE_FIELD_COUNT;

    if (rwAsciiAllocFields(astream, astream->as_field_count)) {
        /* out of memory */
        skAppPrintErr(ERRMSG_ALLOC);
        exit(EXIT_FAILURE);
    }

    for (i = 0; i < astream->as_field_count; ++i) {
        astream->as_field[i].af_field_id = i;
    }
}


/*
 *  rwAsciiVerifyIcmpColumns(astream);
 *
 *    If the ICMP type and code are to be written, verify that the
 *    sPort and dPort columns are present, adding them if required.
 */
static void rwAsciiVerifyIcmpColumns(rwAsciiStream_t *astream)
{
    uint32_t i;
    int have_sport = 0;
    int have_dport = 0;

    for (i = 0; i < astream->as_field_count; ++i) {
        switch (astream->as_field[i].af_field_id) {
          case RWREC_FIELD_SPORT:
            if (have_dport) {
                return;
            }
            have_sport = 1;
            break;

          case RWREC_FIELD_DPORT:
            if (have_sport) {
                return;
            }
            have_dport = 1;
            break;
        }
    }

    /* if we make it here, sport and/or dport are needed */
    if ((astream->as_field_count + 2 - have_dport - have_sport)
        > astream->as_field_capacity)
    {
        if (rwAsciiAllocFields(astream, astream->as_field_count + 2)) {
            /* out of memory */
            skAppPrintErr(ERRMSG_ALLOC);
            exit(EXIT_FAILURE);
        }
    }

    if (!have_sport) {
        astream->as_field[astream->as_field_count].af_field_id
            = RWREC_FIELD_SPORT;
        ++astream->as_field_count;
    }
    if (!have_dport) {
        astream->as_field[astream->as_field_count].af_field_id
            = RWREC_FIELD_DPORT;
        ++astream->as_field_count;
    }
}


/*
 *  rwAsciiPreparePrint(astream);
 *
 *    Do any final initialization prior to printing the column titles
 *    or the first row: Set the field list to the default list if the
 *    caller did not choose specific columns; if ICMP type and code
 *    output was requested, make certain the correct columns exist in
 *    the output; and set the width of the columns.
 */
static void rwAsciiPreparePrint(rwAsciiStream_t *astream)
{
    astream->as_initialized = 1;

    if (astream->as_field_count == 0) {
        rwAsciiSetDefaultFields(astream);
    }

    if (astream->as_legacy_icmp) {
        rwAsciiVerifyIcmpColumns(astream);
    }

    rwAsciiSetWidths(astream);
}


int rwAsciiFlush(rwAsciiStream_t *astream)
{
    return fflush(astream->as_out_stream);
}


void rwAsciiStreamDestroy(rwAsciiStream_t **astream)
{
    assert(astream);

    if (NULL == *astream) {
        return;
    }

    if ((*astream)->as_field) {
        free((*astream)->as_field);
        (*astream)->as_field = NULL;
    }
    free(*astream);
    *astream = NULL;
}


int rwAsciiStreamCreate(rwAsciiStream_t **astream)
{
    assert(astream);

    /* create the object */
    *astream = calloc(1, sizeof(rwAsciiStream_t));
    if (!*astream) {
        /* out of memory */
        skAppPrintErr(ERRMSG_ALLOC);
        return -1;
    }

    /* non-zero defaults */
    (*astream)->as_out_stream = stdout;
    (*astream)->as_delimiter = '|';
#if SK_ENABLE_IPV6
    (*astream)->as_ipv6_policy = SK_IPV6POLICY_MIX;
#else
    (*astream)->as_ipv6_policy = SK_IPV6POLICY_IGNORE;
#endif

    return 0;
}


int rwAsciiAppendOneField(
    rwAsciiStream_t    *astream,
    const uint32_t      field_id)
{
    return rwAsciiAppendFields(astream, &field_id, 1);
}


int rwAsciiAppendFields(
    rwAsciiStream_t    *astream,
    const uint32_t     *field_ids,
    uint32_t            field_count)
{
    uint8_t i;
    int saw_icmp_field = 0;

    assert(astream);

    /* verify input */
    if ((field_count == 0) || (field_ids == NULL)) {
        return -1;
    }

    for (i = 0; i < field_count; ++i) {
        if (field_ids[i] >= RWREC_PRINTABLE_FIELD_COUNT) {
            /* bad value */
            skAppPrintErr(("Value '%" PRIu32 "' is not a value field id"),
                          field_ids[i]);
            return -1;
        }
        if (field_ids[i] == RWREC_FIELD_ICMP_TYPE_CODE) {
            saw_icmp_field = 1;
        }
    }

    /* allocate fields */
    if (astream->as_field_count >= astream->as_field_capacity) {
        if (rwAsciiAllocFields(astream, 0)) {
            /* out of memory */
            skAppPrintErr(ERRMSG_ALLOC);
            return -1;
        }
    }

    for (i = 0; i < field_count; ++i) {
        astream->as_field[astream->as_field_count].af_field_id = field_ids[i];
        ++astream->as_field_count;
    }

    if (saw_icmp_field) {
        astream->as_field_icmp = 1;
    }

    return 0;
}


int rwAsciiAppendCallbackField(
    rwAsciiStream_t            *astream,
    rwAsciiStreamGetTitle_t     get_title_fn,
    rwAsciiStreamGetValue_t     get_value_fn,
    void                       *callback_data,
    uint32_t                    width)
{
    rwascii_field_t *field;

    if (get_title_fn == NULL || get_value_fn == NULL) {
        return -1;
    }

    /* allocate fields */
    if (astream->as_field_count >= astream->as_field_capacity) {
        if (rwAsciiAllocFields(astream, 0)) {
            /* out of memory */
            skAppPrintErr(ERRMSG_ALLOC);
            return -1;
        }
    }

    field = &astream->as_field[astream->as_field_count];
    field->af_field_id = RWASCII_CB_FIELD_ID;
    field->af_cb_gettitle = get_title_fn;
    field->af_cb_getvalue = get_value_fn;
    field->af_cb_data = callback_data;
    field->af_width = width;

    astream->as_field_count++;

    return 0;
}


void rwAsciiSetOutputHandle(rwAsciiStream_t *astream, FILE *fh)
{
    assert(astream);
    if (fh == NULL) {
        astream->as_out_stream = stdout;
    } else {
        astream->as_out_stream = fh;
    }
}


void rwAsciiSetDelimiter(rwAsciiStream_t *astream, char delimiter)
{
    assert(astream);
    astream->as_delimiter = delimiter;
}


void rwAsciiSetNoColumns(rwAsciiStream_t *astream)
{
    assert(astream);
    astream->as_not_columnar = 1;
}


void rwAsciiSetIPv6Policy(rwAsciiStream_t *astream, sk_ipv6policy_t policy)
{
    assert(astream);
    astream->as_ipv6_policy = policy;
}


void rwAsciiSetIntegerIps(rwAsciiStream_t *astream)
{
    assert(astream);

    if (astream->as_zero_pad_ips) {
        skAppPrintErr("Printing IPs as integer overrides zero padding IPs");
        astream->as_zero_pad_ips = 0;
    }

    astream->as_integer_ips = 1;
}


void rwAsciiSetZeroPadIps(rwAsciiStream_t *astream)
{
    assert(astream);

    if (astream->as_integer_ips) {
        skAppPrintErr("Printing IPs as integer overrides zero padding IPs");
        return;
    }

    astream->as_zero_pad_ips = 1;
}


void rwAsciiSetNoTitles(rwAsciiStream_t *astream)
{
    assert(astream);
    astream->as_no_titles = 1;
}


void rwAsciiSetTimestampFlags(rwAsciiStream_t *astream, uint32_t time_flags)
{
    assert(astream);
    astream->as_timeflags = time_flags;
}


void rwAsciiSetIntegerSensors(rwAsciiStream_t *astream)
{
    assert(astream);
    astream->as_integer_sensors = 1;
}


void rwAsciiSetIcmpTypeCode(rwAsciiStream_t *astream)
{
    assert(astream);
    astream->as_legacy_icmp = 1;
}


void rwAsciiSetNoNewline(rwAsciiStream_t *astream)
{
    assert(astream);
    astream->as_no_newline = 1;
}


void rwAsciiSetNoFinalDelimiter(rwAsciiStream_t *astream)
{
    assert(astream);
    astream->as_no_final_delim = 1;
}


void rwAsciiGetFieldName(
    char                       *buf,
    size_t                      buf_len,
    rwrec_printable_fields_t    field_id)
{
    size_t i;

    assert(buf);
    assert(buf_len > 0);

    buf[0] = '\0';

    for (i = 0; i < field_map_count; ++i) {
        if (field_map_entries[i].id == field_id) {
            strncpy(buf, field_map_entries[i].name, buf_len-1);
            buf[buf_len-1] = '\0';
            break;
        }
    }
}


void rwAsciiPrintTitles(rwAsciiStream_t *astream)
{
    const rwascii_field_t *field;
    char buf[128];
    uint32_t i;

    /* initialize */
    if (astream->as_initialized == 0) {
        rwAsciiPreparePrint(astream);
    }

    /* don't print titles if we are not supposed to or if we already
     * have */
    if (astream->as_no_titles) {
        return;
    }
    astream->as_no_titles = 1;

    for (i = 0, field = astream->as_field;
         i < astream->as_field_count;
         ++i, ++field)
    {
        if (i > 0) {
            fprintf(astream->as_out_stream, "%c", astream->as_delimiter);
        }

        if (field->af_field_id == RWREC_FIELD_ICMP_TYPE_CODE) {
            if (astream->as_not_columnar) {
                fprintf(astream->as_out_stream, "%s%c%s",
                        "iType", astream->as_delimiter, "iCode");
            } else {
                fprintf(astream->as_out_stream, "%s%c%s",
                        "iTy", astream->as_delimiter, "iCo");
            }
            continue;
        }

        if (field->af_field_id == RWASCII_CB_FIELD_ID) {
            /* invoke callback */
            field->af_cb_gettitle(buf, sizeof(buf), field->af_cb_data);
        } else {
            rwAsciiGetFieldName(buf, sizeof(buf), field->af_field_id);
        }

        if (astream->as_not_columnar) {
            fprintf(astream->as_out_stream, "%s", buf);
        } else {
            fprintf(astream->as_out_stream, "%*.*s",
                    (int)field->af_width, (int)field->af_width,
                    buf);
        }
    } /* for */

    if ( !astream->as_no_final_delim) {
        fprintf(astream->as_out_stream, "%c", astream->as_delimiter);
    }
    if ( !astream->as_no_newline) {
        fprintf(astream->as_out_stream, "\n");
    }
}


/*  Print 'rwrec' to the ascii stream 'astream'.  See header for details */
void rwAsciiPrintRec(
    rwAsciiStream_t    *astream,
    const rwRec        *rwrec)
{
    static char buffer[128];
    const rwascii_field_t *field;
    skipaddr_t ip;
    uint32_t ip_flags;
    imaxdiv_t idiv;
    uint32_t i;

    assert(astream);
    assert(rwrec);
    assert(sizeof(buffer) > 1+SK_NUM2DOT_STRLEN);
    assert(sizeof(buffer) > 1+SKTIMESTAMP_STRLEN);

    /* initialize */
    if (astream->as_initialized == 0) {
        rwAsciiPreparePrint(astream);
    }

    /* print titles if we haven't */
    if (astream->as_no_titles == 0) {
        /* print titles */
        rwAsciiPrintTitles(astream);
    }

    ip_flags = 0;
    if (astream->as_integer_ips) {
        ip_flags |= SKIPADDR_DECIMAL;
    } else if (astream->as_zero_pad_ips) {
        ip_flags |= SKIPADDR_ZEROPAD;
    }

    for (i = 0, field = astream->as_field;
         i < astream->as_field_count;
         ++i, ++field)
    {
        if (i > 0) {
            fprintf(astream->as_out_stream, "%c", astream->as_delimiter);
        }
        switch (field->af_field_id) {
          case RWREC_FIELD_SIP:
            rwRecMemGetSIP(rwrec, &ip);
            skipaddrString(buffer, &ip, ip_flags);
            break;

          case RWREC_FIELD_DIP:
            rwRecMemGetDIP(rwrec, &ip);
            skipaddrString(buffer, &ip, ip_flags);
            break;

          case RWREC_FIELD_NHIP:
            rwRecMemGetNhIP(rwrec, &ip);
            skipaddrString(buffer, &ip, ip_flags);
            break;

          case RWREC_FIELD_SPORT:
            if (astream->as_legacy_icmp && rwRecIsICMP(rwrec)) {
                /* Put the ICMP type in this column. */
                sprintf(buffer, "%u", (unsigned int)rwRecGetIcmpType(rwrec));
            } else {
                /* Put the sPort value here, regardless of protocol */
                sprintf(buffer, "%u", (unsigned int)rwRecGetSPort(rwrec));
            }
            break;

          case RWREC_FIELD_DPORT:
            if (astream->as_legacy_icmp && rwRecIsICMP(rwrec)) {
                /* Put the ICMP code in this column. */
                sprintf(buffer, "%u", (unsigned int)rwRecGetIcmpCode(rwrec));
            } else {
                /* Put the dPort value here, regardless of protocol */
                sprintf(buffer, "%u", (unsigned int)rwRecGetDPort(rwrec));
            }
            break;

          case RWREC_FIELD_ICMP_TYPE_CODE:
            if (!rwRecIsICMP(rwrec)) {
                /* not ICMP; leave columns blank */
                if (astream->as_not_columnar) {
                    sprintf(buffer, "%c", astream->as_delimiter);
                } else {
                    sprintf(buffer, "%c   ", astream->as_delimiter);
                }
            } else if (astream->as_not_columnar) {
                sprintf(buffer, "%u%c%u",
                        (unsigned int)rwRecGetIcmpType(rwrec),
                        astream->as_delimiter,
                        (unsigned int)rwRecGetIcmpCode(rwrec));
            } else {
                sprintf(buffer, "%u%c%3u",
                        (unsigned int)rwRecGetIcmpType(rwrec),
                        astream->as_delimiter,
                        (unsigned int)rwRecGetIcmpCode(rwrec));
            }
            break;

          case RWREC_FIELD_PROTO:
            sprintf(buffer, "%u", (unsigned int)rwRecGetProto(rwrec));
            break;

          case RWREC_FIELD_PKTS:
            sprintf(buffer, ("%" PRIu32), rwRecGetPkts(rwrec));
            break;

          case RWREC_FIELD_BYTES:
            sprintf(buffer, ("%" PRIu32), rwRecGetBytes(rwrec));
            break;

          case RWREC_FIELD_FLAGS:
            tcpflags_string_r(rwRecGetFlags(rwrec), buffer);
            break;

          case RWREC_FIELD_INIT_FLAGS:
            tcpflags_string_r(rwRecGetInitFlags(rwrec), buffer);
            break;

          case RWREC_FIELD_REST_FLAGS:
            tcpflags_string_r(rwRecGetRestFlags(rwrec), buffer);
            break;

          case RWREC_FIELD_TCP_STATE:
            {
                const uint8_t state = rwRecGetTcpState(rwrec);
                sprintf(buffer, "%c%c%c     ",
                        ((state & SK_TCPSTATE_TIMEOUT_KILLED)
                         ? 'T' : ' '),
                        ((state & SK_TCPSTATE_TIMEOUT_STARTED)
                         ? 'C' : ' '),
                        ((state & SK_TCPSTATE_FIN_FOLLOWED_NOT_ACK)
                         ? 'F' : ' '));
            }
            break;

          case RWREC_FIELD_APPLICATION:
            sprintf(buffer, "%u", (unsigned int)rwRecGetApplication(rwrec));
            break;

          case RWREC_FIELD_ELAPSED:
            if (astream->as_timeflags & SKTIMESTAMP_NOMSEC) {
                sprintf(buffer, ("%" PRIu32), rwRecGetElapsedSeconds(rwrec));
                break;
            }
            /* else fallthough */
          case RWREC_FIELD_ELAPSED_MSEC:
            idiv = imaxdiv(rwRecGetElapsed(rwrec), 1000);
            sprintf(buffer, ("%" PRId64 ".%03" PRId64),
                    idiv.quot, idiv.rem);
            break;

          case RWREC_FIELD_STIME:
            sktimestamp_r(buffer, rwRecGetStartTime(rwrec),
                          astream->as_timeflags);
            break;

          case RWREC_FIELD_STIME_MSEC:
            sktimestamp_r(buffer, rwRecGetStartTime(rwrec),
                          (astream->as_timeflags & ~SKTIMESTAMP_NOMSEC));
            break;

          case RWREC_FIELD_ETIME:
            sktimestamp_r(buffer, rwRecGetEndTime(rwrec),
                          astream->as_timeflags);
            break;

          case RWREC_FIELD_ETIME_MSEC:
            sktimestamp_r(buffer, rwRecGetEndTime(rwrec),
                          (astream->as_timeflags & ~SKTIMESTAMP_NOMSEC));
            break;

          case RWREC_FIELD_SID:
            /* sensor ID */
            if ( !astream->as_integer_sensors ) {
                sksiteSensorGetName(buffer, sizeof(buffer),
                                    rwRecGetSensor(rwrec));
            } else if (SK_INVALID_SENSOR == rwRecGetSensor(rwrec)) {
                strcpy(buffer, "-1");
            } else {
                sprintf(buffer, "%u", (unsigned int)rwRecGetSensor(rwrec));
            }
            break;

          case RWREC_FIELD_INPUT:
            sprintf(buffer, "%u", (unsigned int)rwRecGetInput(rwrec));
            break;

          case RWREC_FIELD_OUTPUT:
            /* output */
            sprintf(buffer, "%u", (unsigned int)rwRecGetOutput(rwrec));
            break;

          case RWREC_FIELD_FTYPE_CLASS:
            sksiteFlowtypeGetClass(buffer, sizeof(buffer),
                                   rwRecGetFlowType(rwrec));
            break;

          case RWREC_FIELD_FTYPE_TYPE:
            sksiteFlowtypeGetType(buffer, sizeof(buffer),
                                  rwRecGetFlowType(rwrec));
            break;

          case RWASCII_CB_FIELD_ID:
            /* invoke callback */
            field->af_cb_getvalue(rwrec, buffer, sizeof(buffer),
                                  field->af_cb_data);
            break;

          default:
            assert(0);
            snprintf(buffer, sizeof(buffer), "BADFIELD");
            break;
        } /* switch */

        if (astream->as_not_columnar) {
            fprintf(astream->as_out_stream, "%s", buffer);
        } else {
            fprintf(astream->as_out_stream, "%*s",
                    (int)field->af_width, buffer);
        }
    } /* for */

    if ( !astream->as_no_final_delim) {
        fprintf(astream->as_out_stream, "%c", astream->as_delimiter);
    }
    if ( !astream->as_no_newline) {
        fprintf(astream->as_out_stream, "\n");
    }

    return;
}
/* rwAsciiPrintRec */


int rwAsciiFieldMapAddDefaultFields(sk_stringmap_t **field_map)
{
    assert(field_map);

    /* Create the map if necessary */
    if (NULL == *field_map) {
        if (SKSTRINGMAP_OK != skStringMapCreate(field_map)) {
            return -1;
        }
    }

    /* add entries */
    if (skStringMapAddEntries(*field_map, field_map_count, field_map_entries)
        != SKSTRINGMAP_OK)
    {
        return -1;
    }

    return 0;
}


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
