#! /usr/bin/perl -w
# MD5: multiple
# TEST: ../rwfilter/rwfilter --stime=2009/02/13:20:00-2009/02/13:20 --sensor=S2 --proto=6 --aport=80,8080,443 --pass=stdout ../../tests/data.rwf | ./rwallformats --basename=sk-teststmp && md5 sk-teststmp*

use strict;
use SiLKTests;

# clean up when we're done
END {
    if (!$ENV{SK_TESTS_SAVEOUTPUT}) {
        # remove files
        unlink glob("$SiLKTests::SILK_TESTS_PREFIX-FT_*dat");
    }
}

my $rwallformats = $ENV{RWALLFORMATS} || './rwallformats';
my $rwfilter = check_silk_app('rwfilter');
my $rwfileinfo = check_silk_app('rwfileinfo');
my %file;
$file{data} = get_data_or_exit77('data');

# remove existing files to avoid overwrite error
unlink glob("$SiLKTests::SILK_TESTS_PREFIX-FT_*dat");

my $cmd = "$rwfilter --stime=2009/02/13:20:00-2009/02/13:20 --sensor=S2 --proto=6 --aport=80,8080,443 --pass=stdout $file{data} | $rwallformats --basename=$SiLKTests::SILK_TESTS_PREFIX";
if (!check_exit_status($cmd)) {
    exit 1;
}

# compute MD5 of each file
while (<DATA>) {
    my ($md5, $file) = split " ";
    if (-f $file) {
        check_md5_file($md5, $file);
    }
}

__DATA__
092d1a092fb43d47e99388c975ec71b9 sk-teststmp-FT_FLOWCAP-v2-c0-B.dat
6764d0068b7d32c5fb581953462a83eb sk-teststmp-FT_FLOWCAP-v2-c0-L.dat
0e72311d1249647a600a15cc6cf50a59 sk-teststmp-FT_FLOWCAP-v2-c1-B.dat
72b66df7abdce8c6038af397d83aaf18 sk-teststmp-FT_FLOWCAP-v2-c1-L.dat
f398a6be7a95ec93cdcdc1b0d0fe3836 sk-teststmp-FT_FLOWCAP-v2-c2-B.dat
16b0d424efa5851bfb813a7f445530f0 sk-teststmp-FT_FLOWCAP-v2-c2-L.dat
2b7aac44cbf0067d39f891c938f15b08 sk-teststmp-FT_FLOWCAP-v3-c0-B.dat
43c0ebef391035ec2cf0f75eff1ad9ad sk-teststmp-FT_FLOWCAP-v3-c0-L.dat
e203b807f6ade9468a74309ae0d291db sk-teststmp-FT_FLOWCAP-v3-c1-B.dat
ac7f1520a6d99b1eb1418f42c6db9320 sk-teststmp-FT_FLOWCAP-v3-c1-L.dat
daa443625d33e700d149ed4e64cc0895 sk-teststmp-FT_FLOWCAP-v3-c2-B.dat
0c2e3eccedd807a7854960cae4044e5a sk-teststmp-FT_FLOWCAP-v3-c2-L.dat
0d215ca58d4b7d7416ebba35d72a6e94 sk-teststmp-FT_FLOWCAP-v4-c0-B.dat
cba6f871223d37102d936fd226413e02 sk-teststmp-FT_FLOWCAP-v4-c0-L.dat
8e577134cdbd53396f34d63b2d9fb53c sk-teststmp-FT_FLOWCAP-v4-c1-B.dat
fa599e2d4283ec8214bc65e51ac65ab3 sk-teststmp-FT_FLOWCAP-v4-c1-L.dat
bda1eb4d6c34d02d1636ef0ed22b2a50 sk-teststmp-FT_FLOWCAP-v4-c2-B.dat
0aad8f4c7266d06ad7a005f3e691a9e7 sk-teststmp-FT_FLOWCAP-v4-c2-L.dat
e9d51b48c1848a4a0976012fd0bac621 sk-teststmp-FT_FLOWCAP-v5-c0-B.dat
551fa7a06f25b71aef0f0e6202245827 sk-teststmp-FT_FLOWCAP-v5-c0-L.dat
0eaa0b6d8eb87e9017898cd73eff40c2 sk-teststmp-FT_FLOWCAP-v5-c1-B.dat
93ccfa23ed0aa8cb2410e2198bd63e0e sk-teststmp-FT_FLOWCAP-v5-c1-L.dat
be1e3d460fd957726e3f5ca057992a57 sk-teststmp-FT_FLOWCAP-v5-c2-B.dat
4008e3b4cfa416c037f303d7431c3870 sk-teststmp-FT_FLOWCAP-v5-c2-L.dat
a156ce595fceefda69878e48de4e4a3a sk-teststmp-FT_RWAUGMENTED-v1-c0-B.dat
9ea12a9dbf321d58d35ab9c8be8216e9 sk-teststmp-FT_RWAUGMENTED-v1-c0-L.dat
cdce2598f6c2bed610c7838cec73fe8f sk-teststmp-FT_RWAUGMENTED-v1-c1-B.dat
734e7e34a48f7543b5d26b438cec30fd sk-teststmp-FT_RWAUGMENTED-v1-c1-L.dat
8bbf9c4757ade0bda9c16b3085d7ce48 sk-teststmp-FT_RWAUGMENTED-v1-c2-B.dat
e614f9a74d0f3ee7c07c6a2a225bfbaf sk-teststmp-FT_RWAUGMENTED-v1-c2-L.dat
10d619e6fe76acd7548b5e2218843e3b sk-teststmp-FT_RWAUGMENTED-v2-c0-B.dat
15fd9f06d6bf9b6023bd3c5177b5d476 sk-teststmp-FT_RWAUGMENTED-v2-c0-L.dat
aa700c7a7039923aa1edf98555bc8bcf sk-teststmp-FT_RWAUGMENTED-v2-c1-B.dat
befdbf5a4318d73df8d01c8b55f72f58 sk-teststmp-FT_RWAUGMENTED-v2-c1-L.dat
509e0f9982cc759cfe6b45670f9dafce sk-teststmp-FT_RWAUGMENTED-v2-c2-B.dat
870ac66c1e721a45ac4647473fe854c9 sk-teststmp-FT_RWAUGMENTED-v2-c2-L.dat
566769862b4375d72168620fd3d6fb05 sk-teststmp-FT_RWAUGMENTED-v3-c0-B.dat
458987864a9f06538bb97a9292f00127 sk-teststmp-FT_RWAUGMENTED-v3-c0-L.dat
a10d410f0333ffa61946ad69fe10a7c0 sk-teststmp-FT_RWAUGMENTED-v3-c1-B.dat
d53794f25be3fa05753b106741cf6cf5 sk-teststmp-FT_RWAUGMENTED-v3-c1-L.dat
40dd3991168a3835ab616a51ad528264 sk-teststmp-FT_RWAUGMENTED-v3-c2-B.dat
605d28bb2771933a3c8f149f86752469 sk-teststmp-FT_RWAUGMENTED-v3-c2-L.dat
df6a5733c0b106ad5018d613ecb365d5 sk-teststmp-FT_RWAUGMENTED-v4-c0-B.dat
a6d38e1eeb6cc8af270e65318992863e sk-teststmp-FT_RWAUGMENTED-v4-c0-L.dat
2624bd18d4a13a88b3e1a6efb5b38d55 sk-teststmp-FT_RWAUGMENTED-v4-c1-B.dat
34175c1f6cbb1d308ee9cc1a94884ebd sk-teststmp-FT_RWAUGMENTED-v4-c1-L.dat
10e770828beb28e304e242ffe714f3c4 sk-teststmp-FT_RWAUGMENTED-v4-c2-B.dat
8389aa7cfa8f4424eefb5861bbe92ad2 sk-teststmp-FT_RWAUGMENTED-v4-c2-L.dat
5a3d601ef8fb6e688f60e506320c7ae7 sk-teststmp-FT_RWAUGROUTING-v1-c0-B.dat
57becc19f40ce0e6424018661b6f9207 sk-teststmp-FT_RWAUGROUTING-v1-c0-L.dat
06301a4325b0261f1dafbddff8936351 sk-teststmp-FT_RWAUGROUTING-v1-c1-B.dat
6da2595e1b81c8ca649366b107e2b176 sk-teststmp-FT_RWAUGROUTING-v1-c1-L.dat
03100f27dd38067728291d9b7032f729 sk-teststmp-FT_RWAUGROUTING-v1-c2-B.dat
d828a5f4e2ea18c6a4f2019b679082a8 sk-teststmp-FT_RWAUGROUTING-v1-c2-L.dat
dfd99a6535477f13d81804c6aa300263 sk-teststmp-FT_RWAUGROUTING-v2-c0-B.dat
049c98340034f445a0a963d98be99af4 sk-teststmp-FT_RWAUGROUTING-v2-c0-L.dat
169cb06a603b9df02bd8df84699ec617 sk-teststmp-FT_RWAUGROUTING-v2-c1-B.dat
71f9195b38fac94960d03b8cf671d9c2 sk-teststmp-FT_RWAUGROUTING-v2-c1-L.dat
2303e1e2c9b01f421534d99f0a569e34 sk-teststmp-FT_RWAUGROUTING-v2-c2-B.dat
38ccaaaf293641c7d5114db1aa99d8d7 sk-teststmp-FT_RWAUGROUTING-v2-c2-L.dat
5de4fa65eda836e0cb658d18c2ef117d sk-teststmp-FT_RWAUGROUTING-v3-c0-B.dat
1657437b2028cb06e0d9cc5af5739d20 sk-teststmp-FT_RWAUGROUTING-v3-c0-L.dat
2f3263cff5b0e6a3a790762b8ecb8528 sk-teststmp-FT_RWAUGROUTING-v3-c1-B.dat
f15bef9ea6818ccfb73b37eac88ca4ec sk-teststmp-FT_RWAUGROUTING-v3-c1-L.dat
3f7aecabd5186df44c12977e26cc0d3f sk-teststmp-FT_RWAUGROUTING-v3-c2-B.dat
6ad101a641554f80fa2380f14f30f22d sk-teststmp-FT_RWAUGROUTING-v3-c2-L.dat
e664817842d0e6797cc2c65084a3a5c7 sk-teststmp-FT_RWAUGROUTING-v4-c0-B.dat
70030029f16aa9b123d6263c81bceab3 sk-teststmp-FT_RWAUGROUTING-v4-c0-L.dat
488ddea657092382cb8ed5e80200579c sk-teststmp-FT_RWAUGROUTING-v4-c1-B.dat
5afd83ca878c885ceaf737f8faf895ff sk-teststmp-FT_RWAUGROUTING-v4-c1-L.dat
6734e04b695887285953ea5f8122a178 sk-teststmp-FT_RWAUGROUTING-v4-c2-B.dat
1833bd59911f7a5cdb0b9655231bca68 sk-teststmp-FT_RWAUGROUTING-v4-c2-L.dat
d3180fdd25ba40b1fe4adef9d77ce46f sk-teststmp-FT_RWAUGSNMPOUT-v1-c0-B.dat
0e87da146403d02f452c41554eb45f03 sk-teststmp-FT_RWAUGSNMPOUT-v1-c0-L.dat
5ebddda3765246210097c8c570aec9c3 sk-teststmp-FT_RWAUGSNMPOUT-v1-c1-B.dat
cc6362fb1e05607d06bff821416c1c6e sk-teststmp-FT_RWAUGSNMPOUT-v1-c1-L.dat
42da5d9a0d4c879fe9bfe9ae9e7ef938 sk-teststmp-FT_RWAUGSNMPOUT-v1-c2-B.dat
d1e05112f783190c53229099a4939dd1 sk-teststmp-FT_RWAUGSNMPOUT-v1-c2-L.dat
05ea7fd4ecaa58860a9597d7b32d9af3 sk-teststmp-FT_RWAUGSNMPOUT-v2-c0-B.dat
75be5e301f53e6ad0badc0f3b64db1ce sk-teststmp-FT_RWAUGSNMPOUT-v2-c0-L.dat
b93432c5667b2580994084d18bbed27f sk-teststmp-FT_RWAUGSNMPOUT-v2-c1-B.dat
b54dbe8e2f3fbfb5968087f051ffbe46 sk-teststmp-FT_RWAUGSNMPOUT-v2-c1-L.dat
783e919ff631e2a912516b3ca8294ad6 sk-teststmp-FT_RWAUGSNMPOUT-v2-c2-B.dat
6e530898fd617f55f47f32e50d935b9e sk-teststmp-FT_RWAUGSNMPOUT-v2-c2-L.dat
ee551994da88df5b5747f1bcda83af7a sk-teststmp-FT_RWAUGSNMPOUT-v3-c0-B.dat
7bbe5cb5939d558774e0f8f1dbe15ba8 sk-teststmp-FT_RWAUGSNMPOUT-v3-c0-L.dat
f4220ae686ba3534026ba740fd6097eb sk-teststmp-FT_RWAUGSNMPOUT-v3-c1-B.dat
348d50a2a5587efb37e665f18429f5e8 sk-teststmp-FT_RWAUGSNMPOUT-v3-c1-L.dat
6a0aa56576888a430f381cfb94f7d05d sk-teststmp-FT_RWAUGSNMPOUT-v3-c2-B.dat
ad562a3b61643d049032712866a07402 sk-teststmp-FT_RWAUGSNMPOUT-v3-c2-L.dat
5c0c587aa2c393f69a17f300444ef313 sk-teststmp-FT_RWAUGSNMPOUT-v4-c0-B.dat
2b5cae4246d18668a7cb2955e2edac81 sk-teststmp-FT_RWAUGSNMPOUT-v4-c0-L.dat
b40883b87d9dd15e9128921fd0932e23 sk-teststmp-FT_RWAUGSNMPOUT-v4-c1-B.dat
0dbc4852de0a9d361e255ee16f86dac2 sk-teststmp-FT_RWAUGSNMPOUT-v4-c1-L.dat
068490286f68f15194aa3057dfac1824 sk-teststmp-FT_RWAUGSNMPOUT-v4-c2-B.dat
516693b5f1a0bbd0b59fe0de7cbedb6d sk-teststmp-FT_RWAUGSNMPOUT-v4-c2-L.dat
7e97ac6fbbf56fc8168588b03299c1dc sk-teststmp-FT_RWAUGWEB-v1-c0-B.dat
bd126e6e8867c722a6a9df4d482bcbfb sk-teststmp-FT_RWAUGWEB-v1-c0-L.dat
92d54a252517439f54b04085917b5561 sk-teststmp-FT_RWAUGWEB-v1-c1-B.dat
9e287c6bba0b069311553b62fe03bd2a sk-teststmp-FT_RWAUGWEB-v1-c1-L.dat
022ba409ad9c3654e4373b2d0e7a18c8 sk-teststmp-FT_RWAUGWEB-v1-c2-B.dat
8d78f7696eddd3e2a1bdfe831a1386cb sk-teststmp-FT_RWAUGWEB-v1-c2-L.dat
a93b0b429b0691a79651d42f9a0ff39c sk-teststmp-FT_RWAUGWEB-v2-c0-B.dat
20e51bee874e648268767d9e1b53e375 sk-teststmp-FT_RWAUGWEB-v2-c0-L.dat
29c86b34814395bc39957b4f27015798 sk-teststmp-FT_RWAUGWEB-v2-c1-B.dat
74022c3a161c097c3f75e12ea8a01d5d sk-teststmp-FT_RWAUGWEB-v2-c1-L.dat
43f7209ed85dbed055be0ccf317d282b sk-teststmp-FT_RWAUGWEB-v2-c2-B.dat
872c02db3ed0471edd9c7fb177d4a3ee sk-teststmp-FT_RWAUGWEB-v2-c2-L.dat
0471bfcccb9b2298d6f9cd12e5c226f9 sk-teststmp-FT_RWAUGWEB-v3-c0-B.dat
bd3b8e38e7919eb6003fa23239d988b8 sk-teststmp-FT_RWAUGWEB-v3-c0-L.dat
9884f6912a642a6bc8ac70c63278184e sk-teststmp-FT_RWAUGWEB-v3-c1-B.dat
256db7d90961ffec59a4a71ef1699305 sk-teststmp-FT_RWAUGWEB-v3-c1-L.dat
2b23b328ba96ce4ddb8aaae47c0c4e02 sk-teststmp-FT_RWAUGWEB-v3-c2-B.dat
b7e63cf85f1b85c8c23dde2c1f202958 sk-teststmp-FT_RWAUGWEB-v3-c2-L.dat
2e9b734c6441a74c5294dca272e00cf2 sk-teststmp-FT_RWAUGWEB-v4-c0-B.dat
3c81d6c9d36b5addab4b79ac27abecc6 sk-teststmp-FT_RWAUGWEB-v4-c0-L.dat
e044fc52fc873387c63c2a96dc5deeef sk-teststmp-FT_RWAUGWEB-v4-c1-B.dat
099dfc300784680d2edc577cb84eba8e sk-teststmp-FT_RWAUGWEB-v4-c1-L.dat
df46b68c0bc1252c27d9ccdf4ab57743 sk-teststmp-FT_RWAUGWEB-v4-c2-B.dat
26a72016587f6261918e51cd16c143bc sk-teststmp-FT_RWAUGWEB-v4-c2-L.dat
d0c20aa8cd0c20e5f0685a27c37d6573 sk-teststmp-FT_RWFILTER-v1-c0-B.dat
063c7131b2e0f71b9abbc5634b81a9b6 sk-teststmp-FT_RWFILTER-v1-c0-L.dat
1431430687a808f3facc7cc28f823613 sk-teststmp-FT_RWFILTER-v1-c1-B.dat
0fbfae3d8c6d9b4b953e4dc340f6fc9c sk-teststmp-FT_RWFILTER-v1-c1-L.dat
5f2b31ba66b72e94f072d7de7154bc4b sk-teststmp-FT_RWFILTER-v1-c2-B.dat
240b74538c31f88c8e0938357bc76418 sk-teststmp-FT_RWFILTER-v1-c2-L.dat
8cf8ebbb7f4000545194fc36b2f112e4 sk-teststmp-FT_RWFILTER-v2-c0-B.dat
d16c202ecc787c005d7d97b26a1f7916 sk-teststmp-FT_RWFILTER-v2-c0-L.dat
4a4805d287bcc0b563754a2609cec766 sk-teststmp-FT_RWFILTER-v2-c1-B.dat
6a6e7fdbac7bafbad5b275725fff16ab sk-teststmp-FT_RWFILTER-v2-c1-L.dat
1dd0ddd572c10c002fe33d8b6b92f6b7 sk-teststmp-FT_RWFILTER-v2-c2-B.dat
7a7f31e2871f38e889bd66518acbf5df sk-teststmp-FT_RWFILTER-v2-c2-L.dat
6c3d222edad5884353873c62bfda1bc2 sk-teststmp-FT_RWFILTER-v3-c0-B.dat
1df687ab9689bbea39a542344902651a sk-teststmp-FT_RWFILTER-v3-c0-L.dat
3092d17942600676b13ecafc32762420 sk-teststmp-FT_RWFILTER-v3-c1-B.dat
4cd03fd6e17f17f6ad35b13085574afc sk-teststmp-FT_RWFILTER-v3-c1-L.dat
d8e7bc2fdb5e0f63d1930c5c4044ce81 sk-teststmp-FT_RWFILTER-v3-c2-B.dat
4da73d69d21c90609425fdfab1262c75 sk-teststmp-FT_RWFILTER-v3-c2-L.dat
ea725f206d89c1c7d70c75ea020ce697 sk-teststmp-FT_RWFILTER-v4-c0-B.dat
0d3abe9f42d6ad81049981e499fd4458 sk-teststmp-FT_RWFILTER-v4-c0-L.dat
77242ea6805ac33937b406e200d8fe71 sk-teststmp-FT_RWFILTER-v4-c1-B.dat
be7ba781fd88d97685fcd9edbfe85d20 sk-teststmp-FT_RWFILTER-v4-c1-L.dat
42054effe5f553298160f53feb0c404e sk-teststmp-FT_RWFILTER-v4-c2-B.dat
9789a65e843bacd7c9d829f12497cf4d sk-teststmp-FT_RWFILTER-v4-c2-L.dat
86a9220018d955d0562a1f838f51e194 sk-teststmp-FT_RWFILTER-v5-c0-B.dat
5657b317f6f8fc8a42d6a3fc2c3c719c sk-teststmp-FT_RWFILTER-v5-c0-L.dat
020a9de4ea9847f67ccf0b9ccd9e01ee sk-teststmp-FT_RWFILTER-v5-c1-B.dat
e811bc9c5b4b4cac12f39995b2ffc1dd sk-teststmp-FT_RWFILTER-v5-c1-L.dat
e7c6977808303b3d2d62eeb1fb1d241e sk-teststmp-FT_RWFILTER-v5-c2-B.dat
d83da05f566c53678a351384b63ef994 sk-teststmp-FT_RWFILTER-v5-c2-L.dat
4c247ebf27eff77284f555f6488254d9 sk-teststmp-FT_RWGENERIC-v0-c0-B.dat
78399f6b947023756cd46941450b44d5 sk-teststmp-FT_RWGENERIC-v0-c0-L.dat
c8409410bae97b61b6796ac15e8b33ae sk-teststmp-FT_RWGENERIC-v0-c1-B.dat
704e349f3c5b3164dfc83024badbe044 sk-teststmp-FT_RWGENERIC-v0-c1-L.dat
e8fd45b2eadc6161cb9da9ce19f8223c sk-teststmp-FT_RWGENERIC-v0-c2-B.dat
7ed0e6d1058873229655e2d8702edd57 sk-teststmp-FT_RWGENERIC-v0-c2-L.dat
44cd61b49ae99553d06d13b6f39bc107 sk-teststmp-FT_RWGENERIC-v1-c0-B.dat
c4685938d0d3c85cdb2feedbd8b86cd9 sk-teststmp-FT_RWGENERIC-v1-c0-L.dat
4e09693db4b2940a55a0f6a8c582ac66 sk-teststmp-FT_RWGENERIC-v1-c1-B.dat
d97f756ad9c756b635769ec7e91b00c7 sk-teststmp-FT_RWGENERIC-v1-c1-L.dat
064a810677130d2abc8ecb88e9116629 sk-teststmp-FT_RWGENERIC-v1-c2-B.dat
6387d479c49c74e30c1624fadd7b1748 sk-teststmp-FT_RWGENERIC-v1-c2-L.dat
6f788126ba9e9a8fd37090713b3ec302 sk-teststmp-FT_RWGENERIC-v2-c0-B.dat
f706d36086f59813ef4b657dbca8887d sk-teststmp-FT_RWGENERIC-v2-c0-L.dat
9d172524c34f8c521332ae3188253a15 sk-teststmp-FT_RWGENERIC-v2-c1-B.dat
f1bb47584c1a9d1d707de179f0f28a97 sk-teststmp-FT_RWGENERIC-v2-c1-L.dat
287ecfc08c9a4e4a496744a8feaa5332 sk-teststmp-FT_RWGENERIC-v2-c2-B.dat
b48062e6cb03973dd92dc2ebafd07dc2 sk-teststmp-FT_RWGENERIC-v2-c2-L.dat
692a7fa27f757f36219064debb847923 sk-teststmp-FT_RWGENERIC-v3-c0-B.dat
dd6d31a28ff002b8a8571d41c4fa9b0d sk-teststmp-FT_RWGENERIC-v3-c0-L.dat
70df9cc26e29d363e1f8e91cd5ccb8d2 sk-teststmp-FT_RWGENERIC-v3-c1-B.dat
6c65a10877e12d7062b357ed3214119d sk-teststmp-FT_RWGENERIC-v3-c1-L.dat
a397d4ddf9f2bf227be5e78015b13d05 sk-teststmp-FT_RWGENERIC-v3-c2-B.dat
e3c4c5296c12f039b5ea658ec1bddb90 sk-teststmp-FT_RWGENERIC-v3-c2-L.dat
53d9fdf36b2195abc062221726d502a5 sk-teststmp-FT_RWGENERIC-v4-c0-B.dat
0976b290a3c03768e618131119be4d2b sk-teststmp-FT_RWGENERIC-v4-c0-L.dat
504e59bf1363a0d9a2084c9fb552b7d3 sk-teststmp-FT_RWGENERIC-v4-c1-B.dat
0a3023be5e6c5c37a21db2315873b28d sk-teststmp-FT_RWGENERIC-v4-c1-L.dat
6329752c74511ec877292c8a158b6183 sk-teststmp-FT_RWGENERIC-v4-c2-B.dat
581380102dafb69e94dc2b117bf27c71 sk-teststmp-FT_RWGENERIC-v4-c2-L.dat
81542ee0576ff1ab801ec67228e743ea sk-teststmp-FT_RWGENERIC-v5-c0-B.dat
9106c7c40d8ae28d7957fa4ffc911ded sk-teststmp-FT_RWGENERIC-v5-c0-L.dat
b84752f64c28699c209df5c7cbf25af8 sk-teststmp-FT_RWGENERIC-v5-c1-B.dat
8598e228deb17c5bff349e7c9628155d sk-teststmp-FT_RWGENERIC-v5-c1-L.dat
7f83b580447bb367b135e3177b8f6e95 sk-teststmp-FT_RWGENERIC-v5-c2-B.dat
13b5d469c2bda661ea399f4ff7bbce83 sk-teststmp-FT_RWGENERIC-v5-c2-L.dat
4d78fa5c6703853b9376c83c2bc757ef sk-teststmp-FT_RWIPV6-v1-c0-B.dat
0fa32caa9c2e72fbf70bf3abeaf34332 sk-teststmp-FT_RWIPV6-v1-c0-L.dat
c4f68c80c72f1d8cf70ede927b01b06f sk-teststmp-FT_RWIPV6-v1-c1-B.dat
2def32a4a35bea1e9540e9a3f08e29b3 sk-teststmp-FT_RWIPV6-v1-c1-L.dat
29baf6f41c75abc70cb54928f6ea52ca sk-teststmp-FT_RWIPV6-v1-c2-B.dat
11123e2c09d1e334accf216dbcb1444f sk-teststmp-FT_RWIPV6-v1-c2-L.dat
7db0e19df62db2cabbe821a9905343a2 sk-teststmp-FT_RWIPV6ROUTING-v1-c0-B.dat
44560c373f6f5b91d8685e4148d7d2b5 sk-teststmp-FT_RWIPV6ROUTING-v1-c0-L.dat
82b9824ca15942ce826f67473bb474b9 sk-teststmp-FT_RWIPV6ROUTING-v1-c1-B.dat
59dde37dc63ccc7a557d8e4f5bb12d2f sk-teststmp-FT_RWIPV6ROUTING-v1-c1-L.dat
035157348793a49dfe57ffeaac975d8f sk-teststmp-FT_RWIPV6ROUTING-v1-c2-B.dat
04e1f84c607dae6e4b9bb57dd9813425 sk-teststmp-FT_RWIPV6ROUTING-v1-c2-L.dat
210d93b9a1950c903c76257c8d29eae8 sk-teststmp-FT_RWNOTROUTED-v1-c0-B.dat
eb90aa498784a13143d858ed07510303 sk-teststmp-FT_RWNOTROUTED-v1-c0-L.dat
8c959c48949c90b9bacc018a481878be sk-teststmp-FT_RWNOTROUTED-v1-c1-B.dat
79d6ccc2fb9636289b7813c6e7d0b41b sk-teststmp-FT_RWNOTROUTED-v1-c1-L.dat
10355ee84fcc2f043ae3c96ad4728384 sk-teststmp-FT_RWNOTROUTED-v1-c2-B.dat
9e3e859fa9c3f4b5f0308acc8dcd4ca7 sk-teststmp-FT_RWNOTROUTED-v1-c2-L.dat
eb5e675767ca70835ffa06f7425d453d sk-teststmp-FT_RWNOTROUTED-v2-c0-B.dat
37c734a1d0e02c2e3d398fe05015c1a0 sk-teststmp-FT_RWNOTROUTED-v2-c0-L.dat
23145b41d97d112065fe9a475bf10082 sk-teststmp-FT_RWNOTROUTED-v2-c1-B.dat
11da44696b6aa6dcdd05a0016b415767 sk-teststmp-FT_RWNOTROUTED-v2-c1-L.dat
8403f42693b843a629efc9f7505d76fe sk-teststmp-FT_RWNOTROUTED-v2-c2-B.dat
5a456b103db4f8c3667cef1f37d8015c sk-teststmp-FT_RWNOTROUTED-v2-c2-L.dat
5045e5af0ae2c39c63e6383f529b41dd sk-teststmp-FT_RWNOTROUTED-v3-c0-B.dat
c9be3a20490f288a53f43ebd4573cadb sk-teststmp-FT_RWNOTROUTED-v3-c0-L.dat
af4659b9394728b8bd8ff2b7fc967764 sk-teststmp-FT_RWNOTROUTED-v3-c1-B.dat
d6fc6523f9a7655e9b475b5260897ad6 sk-teststmp-FT_RWNOTROUTED-v3-c1-L.dat
de8f656e7c495fe3544196ea0b3cbd1b sk-teststmp-FT_RWNOTROUTED-v3-c2-B.dat
d5b176abfaa465a723e6586bcfbb460a sk-teststmp-FT_RWNOTROUTED-v3-c2-L.dat
32687f0032d55bba4f1de98253300c6d sk-teststmp-FT_RWNOTROUTED-v4-c0-B.dat
e3c58d0f875f74620d6d7a49dc691841 sk-teststmp-FT_RWNOTROUTED-v4-c0-L.dat
c18557f0dc614a50412c46d34e51fead sk-teststmp-FT_RWNOTROUTED-v4-c1-B.dat
ff67e70f95fd55431955f49dfd96731f sk-teststmp-FT_RWNOTROUTED-v4-c1-L.dat
384425731fccbb527f033edc5da081ea sk-teststmp-FT_RWNOTROUTED-v4-c2-B.dat
26ce3a995c207c24e4b0bf5b08c79267 sk-teststmp-FT_RWNOTROUTED-v4-c2-L.dat
859774e84685eace2237884936b0dcdf sk-teststmp-FT_RWNOTROUTED-v5-c0-B.dat
3d27e59dafa37824ef777c7eca29b0f3 sk-teststmp-FT_RWNOTROUTED-v5-c0-L.dat
44242a3133d6f00df568d42da78be602 sk-teststmp-FT_RWNOTROUTED-v5-c1-B.dat
3ad7e9f58ce63fe45fcc778e8fdaaae1 sk-teststmp-FT_RWNOTROUTED-v5-c1-L.dat
ea606833942eb09d8050afff72095fa7 sk-teststmp-FT_RWNOTROUTED-v5-c2-B.dat
4772ce7c867962951a6701692f7c1d00 sk-teststmp-FT_RWNOTROUTED-v5-c2-L.dat
3b76f9f75c3914d47f1c86604a4ad517 sk-teststmp-FT_RWROUTED-v1-c0-B.dat
a0ebf3cb025cc587469903d5896a135f sk-teststmp-FT_RWROUTED-v1-c0-L.dat
d2e616b6de2b7ab646d834342dbff87f sk-teststmp-FT_RWROUTED-v1-c1-B.dat
ff828abf3340aef8bb7d38a44c1cc5b4 sk-teststmp-FT_RWROUTED-v1-c1-L.dat
30f36fe306960cd81cd72fe2552d946a sk-teststmp-FT_RWROUTED-v1-c2-B.dat
8b107bcb0fa9aeaf441e17e68d0535e6 sk-teststmp-FT_RWROUTED-v1-c2-L.dat
148897a4893d9113af488631db408285 sk-teststmp-FT_RWROUTED-v2-c0-B.dat
5a886ffbb1e805a77451551bff334688 sk-teststmp-FT_RWROUTED-v2-c0-L.dat
e737d4ed105b4e3309d37113d86b61f6 sk-teststmp-FT_RWROUTED-v2-c1-B.dat
bedc5ac086bda047ed7e03256a25b986 sk-teststmp-FT_RWROUTED-v2-c1-L.dat
1953d71321a8c886eca3da34e5844f66 sk-teststmp-FT_RWROUTED-v2-c2-B.dat
2f6c36cf6623134ce8e85cd55e834dd2 sk-teststmp-FT_RWROUTED-v2-c2-L.dat
c57ae3e5a7aedab40a7dcd9975071495 sk-teststmp-FT_RWROUTED-v3-c0-B.dat
63cebcdf52d245a7fe49af769a90e16e sk-teststmp-FT_RWROUTED-v3-c0-L.dat
b6fad830bd739a297798b4c7020266a5 sk-teststmp-FT_RWROUTED-v3-c1-B.dat
f20d66eaed352d11535ccb7a7cc30954 sk-teststmp-FT_RWROUTED-v3-c1-L.dat
84390548814da5eaf85a6371d435ae9a sk-teststmp-FT_RWROUTED-v3-c2-B.dat
006c21e39f05397838f8be686c44b5ce sk-teststmp-FT_RWROUTED-v3-c2-L.dat
a86316f6eadee4940b4a87aa86ea9bda sk-teststmp-FT_RWROUTED-v4-c0-B.dat
dc9b556a6d8d358b68e4585d4aab76e0 sk-teststmp-FT_RWROUTED-v4-c0-L.dat
043b5bdefccf5c14a0c94797f54f355d sk-teststmp-FT_RWROUTED-v4-c1-B.dat
16b0f526a2ea599f754587dad64e4efb sk-teststmp-FT_RWROUTED-v4-c1-L.dat
b1a02d3078e0ba129fe8ecc427e3f17e sk-teststmp-FT_RWROUTED-v4-c2-B.dat
a8f732eff99d556670139df2bc820a25 sk-teststmp-FT_RWROUTED-v4-c2-L.dat
ee9ec306ffe4ed42dc91afb3fadd141c sk-teststmp-FT_RWROUTED-v5-c0-B.dat
8d56fc47aaefa3af4577306388fd070d sk-teststmp-FT_RWROUTED-v5-c0-L.dat
556230d675c5a71867e03faf0ef20137 sk-teststmp-FT_RWROUTED-v5-c1-B.dat
ec82c9041a39bac689a87fcf1ca21320 sk-teststmp-FT_RWROUTED-v5-c1-L.dat
7cc6cbb667365ccbd7b111191fd12e89 sk-teststmp-FT_RWROUTED-v5-c2-B.dat
25ba28b8879c8e486fec939ea4e6ab59 sk-teststmp-FT_RWROUTED-v5-c2-L.dat
3e722180f6ae96f2af9e48bb99e34af3 sk-teststmp-FT_RWSPLIT-v1-c0-B.dat
6f77510d1ab2f85f975dd4af0b2927ff sk-teststmp-FT_RWSPLIT-v1-c0-L.dat
883af9cbe83cc92571ff8bb6305ebb08 sk-teststmp-FT_RWSPLIT-v1-c1-B.dat
04afbe7ae9b15a261cd06159594ca295 sk-teststmp-FT_RWSPLIT-v1-c1-L.dat
7e01fff58f3fd78967157c2505943a58 sk-teststmp-FT_RWSPLIT-v1-c2-B.dat
259c381882f8a6e5cce73325350bd45a sk-teststmp-FT_RWSPLIT-v1-c2-L.dat
662ed17901932356e274596285342fe1 sk-teststmp-FT_RWSPLIT-v2-c0-B.dat
73fec6532775a7515d7b40769a8f843e sk-teststmp-FT_RWSPLIT-v2-c0-L.dat
783586991fa4d77587c91bed47cfb6c8 sk-teststmp-FT_RWSPLIT-v2-c1-B.dat
264a1ab132a90f8f1f434dbda6e4156b sk-teststmp-FT_RWSPLIT-v2-c1-L.dat
99d23de90690415b8c8b7fad23206c5c sk-teststmp-FT_RWSPLIT-v2-c2-B.dat
f4d6ffa18cb01c66bf7ab95da0e56e17 sk-teststmp-FT_RWSPLIT-v2-c2-L.dat
0f051369d0269c749d3c9435fe29e8b2 sk-teststmp-FT_RWSPLIT-v3-c0-B.dat
0f7b94d46b871e715e0946aa2d05b701 sk-teststmp-FT_RWSPLIT-v3-c0-L.dat
0d7a4520c718a6ab90be29574789cd46 sk-teststmp-FT_RWSPLIT-v3-c1-B.dat
1e42e35c91d342fa6fb486a960c9e336 sk-teststmp-FT_RWSPLIT-v3-c1-L.dat
35815f74d4efef6ec47cd3fcc332334b sk-teststmp-FT_RWSPLIT-v3-c2-B.dat
8a02d44138d79face3965c9bf35b7e6b sk-teststmp-FT_RWSPLIT-v3-c2-L.dat
ca0feeeb1db14fe3fa63131ca6e65628 sk-teststmp-FT_RWSPLIT-v4-c0-B.dat
907cb78f367623588e7b48222c01eceb sk-teststmp-FT_RWSPLIT-v4-c0-L.dat
c1481b3963ba5e70a8e3458988cef8df sk-teststmp-FT_RWSPLIT-v4-c1-B.dat
bc4951ad3cc5ca6106d9ae5bab5dac43 sk-teststmp-FT_RWSPLIT-v4-c1-L.dat
0d7c24f386c2f18665e09d8e0f17a8bf sk-teststmp-FT_RWSPLIT-v4-c2-B.dat
9501622425e9ec5b2faa88ad0ce33983 sk-teststmp-FT_RWSPLIT-v4-c2-L.dat
68d12cb87ad6f65fb88a68c0e6e4503d sk-teststmp-FT_RWSPLIT-v5-c0-B.dat
c5ab9dc6a6db922d3d08b8158553667e sk-teststmp-FT_RWSPLIT-v5-c0-L.dat
b86225013a90436ff54a07df29525d3a sk-teststmp-FT_RWSPLIT-v5-c1-B.dat
561c28c8f1110c11c5be777e6adb79dd sk-teststmp-FT_RWSPLIT-v5-c1-L.dat
bfef66868619e13d07c840e1191f288c sk-teststmp-FT_RWSPLIT-v5-c2-B.dat
4404f32bdd0cc2c664587a43011bd02d sk-teststmp-FT_RWSPLIT-v5-c2-L.dat
6a90cc2561121857c3a08f002c3fe11e sk-teststmp-FT_RWWWW-v1-c0-B.dat
eb6f950e14eb0890854885c90f7973f2 sk-teststmp-FT_RWWWW-v1-c0-L.dat
ad199b8ca62a5171dd7ddca1bb644acf sk-teststmp-FT_RWWWW-v1-c1-B.dat
db8d7d8a1f29edb018636be346531ca3 sk-teststmp-FT_RWWWW-v1-c1-L.dat
2bb6b8ece58ade587115ab76529f66e8 sk-teststmp-FT_RWWWW-v1-c2-B.dat
bd0277d4d2b8502b4ef83fee383a9828 sk-teststmp-FT_RWWWW-v1-c2-L.dat
6eb27bca40fe19f3cfa6b7b7f27cd705 sk-teststmp-FT_RWWWW-v2-c0-B.dat
68f914b49f50944a651f67b631d42ce8 sk-teststmp-FT_RWWWW-v2-c0-L.dat
a0737031cfdbe9e752ca3cf8de2116a5 sk-teststmp-FT_RWWWW-v2-c1-B.dat
4ee0a51098fba2379e5cd26dc7b4e842 sk-teststmp-FT_RWWWW-v2-c1-L.dat
46c2057d78ea4f723c5395c950f92c0e sk-teststmp-FT_RWWWW-v2-c2-B.dat
dc63d9a98f556de31492c3d2ca07cafc sk-teststmp-FT_RWWWW-v2-c2-L.dat
18eed01adb71146160e3ffc16c9e4df7 sk-teststmp-FT_RWWWW-v3-c0-B.dat
24ff54d957e93a3db553a4a3306b30b1 sk-teststmp-FT_RWWWW-v3-c0-L.dat
9db52dbf20dba635dd94fa121c620780 sk-teststmp-FT_RWWWW-v3-c1-B.dat
e4c020b123f84a27fa61fc20d70e2968 sk-teststmp-FT_RWWWW-v3-c1-L.dat
2d68e76dcddf2d72ed2ca6470b0c3ac9 sk-teststmp-FT_RWWWW-v3-c2-B.dat
985da0ff6ea53003536c4d86a7247e19 sk-teststmp-FT_RWWWW-v3-c2-L.dat
58afbca2c7128968ae8f9eb8c7aceb64 sk-teststmp-FT_RWWWW-v4-c0-B.dat
22291fb443bfbad20de3998003164fd8 sk-teststmp-FT_RWWWW-v4-c0-L.dat
10f69da75de1fddc9bdb713ac2d64f5f sk-teststmp-FT_RWWWW-v4-c1-B.dat
e045b8ecb68c49439ca0490e9fa314ff sk-teststmp-FT_RWWWW-v4-c1-L.dat
bab0428bc2e9526492869e72138cc817 sk-teststmp-FT_RWWWW-v4-c2-B.dat
84b1451fa780e80955ea6173c892cb48 sk-teststmp-FT_RWWWW-v4-c2-L.dat
54746495777f62f015b2fcd4a6a2d504 sk-teststmp-FT_RWWWW-v5-c0-B.dat
ca6782765d0991aa42ef183f12a70976 sk-teststmp-FT_RWWWW-v5-c0-L.dat
33938104f2564a0c36e6acb0427731b3 sk-teststmp-FT_RWWWW-v5-c1-B.dat
6f90836356e1498d271be96246ba260b sk-teststmp-FT_RWWWW-v5-c1-L.dat
6bf1f2e1095bee6b64b57dae74e9727e sk-teststmp-FT_RWWWW-v5-c2-B.dat
a22ca00da9ecb142fe8841ceda81f5b1 sk-teststmp-FT_RWWWW-v5-c2-L.dat
