/*
** Copyright (C) 2001-2010 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWUNIQTEMP_H
#define _RWUNIQTEMP_H 1

#include <silk/silk.h>

RCSIDENTVAR(rcsID_RWSORTUTILS_H, "$SiLK: rwuniqtemp.h 15441 2010-03-10 15:58:28Z mthomas $");

/*
**  rwuniqtemp.h
**
**    Functions to handle temp file creation and access.
**
*/



/* LOCAL DEFINES AND TYPEDEFS */

/*
 *  For debugging.  Argument should be surrounded by double-parens:
 *      PRINTDEBUG((stderr, "My output\n"));
 */
#ifdef ENABLE_PRINTDEBUG
#  define PRINTDEBUG(args) fprintf args
#endif
#ifndef PRINTDEBUG
#  define PRINTDEBUG(args)
#endif

/*
 *  Maximum number of temporary files to create.  We need an array of
 *  this many char*'s to point to the file names.
 */
#define MAX_TMP_FILES  2048

/*
 *  Where to create temp files by default.  This can be overridden by
 *  the --temp-dir switch, or environment variable named in
 *  TMPDIR_ENVAR1 or the environment variable named in TMPDIR2.
 *
 *  If this is not defined, no default exists.
 */
#define DEFAULT_TEMP_DIR "/tmp"

/*
 *  Name of primary envionment variable that holds name of temp
 *  directory.  This is consulted when the --temp-dir switch is not
 *  given.
 */
#define TMPDIR_ENVAR1 "SILK_TMPDIR"

/*
 *  Name of alternate envionment variable that holds name of temp
 *  directory.  Used when the --temp-dir switch is not given and the
 *  variable named by TMPDIR_ENVAR1 is not set.
 */
#define TMPDIR_ENVAR2 "TMPDIR"

/*
 *  Function type for signal handler
 */
typedef void (*sig_fn_t)(int);


int appSetSignalHandler(sig_fn_t);
void tempFileSetTempdir(const char *temp_dir);
int tempFileCheckTempdir(void);
const char *tempFileGetName(int tmp_idx);
FILE *tempFileCreate(int tmp_idx);
FILE *tempFileOpen(int tmp_idx);
void tempFileRemove(int tmp_idx);
void tempFileRemoveAll(void);
int tempFileDumpRecords(
    int             tmp_idx,
    const void     *rec_buffer,
    uint32_t        rec_size,
    uint32_t        rec_count);

#endif  /* _RWUNIQTEMP_H */

/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
